__author__ = "hugo.inzirillo"

from napoleontoolbox.bifrost.connectors.position_service import NapoleonPositionServiceConnector
from napoleontoolbox.bifrost.connectors.quote_service import NapoleonQuoteServiceConnector
from napoleontoolbox.bifrost.models.auth import PasswordCredentials, ClientCredentials, Token, Scope
from napoleontoolbox.bifrost.models.filter import SignalComputationFilter, EodQuoteFilter
from napoleontoolbox.bifrost.models.signal import SignalComputation

__all__ = ['PasswordCredentials',
           'ClientCredentials',
           'Token',
           'Scope',
           'NapoleonPositionServiceConnector',
           'NapoleonQuoteServiceConnector',
           'SignalComputationFilter',
           'EodQuoteFilter',
           'SignalComputation']



__author__ = "hugo.inzirillo"

from typing import List

from napoleontoolbox.bifrost.connectors.napoleon_connector import NapoleonConnectorManager
from napoleontoolbox.bifrost.headers import TemplateConnector
from napoleontoolbox.bifrost.models.filter import SignalComputationFilter, EodQuoteFilter
from napoleontoolbox.bifrost.utils.rest_template import Post


class NapoleonQuoteServiceConnector(TemplateConnector):
    def __init__(self):
        self.__connector_manager = NapoleonConnectorManager()
        self.__connector_manager.url = "https://api.napoleonx.ai/position-service/v1/signal-computation"

    @property
    def connector_manager(self):
        return self.__connector_manager

    def authenticate(self):
        self.connector_manager.authentication_manager.authenticate()

    @Post("/filter")
    def filter(self, data: EodQuoteFilter = None) -> List[dict]:
        """

        Parameters
        ----------
        data
        kwargs

        Returns
        -------

        """
        pass


if __name__ == '__main__':
    test = NapoleonQuoteServiceConnector()


__author__ = "hugo.inzirillo"

import json
from abc import abstractmethod, ABCMeta

from pandas import Timestamp


class AbstractModelParamTemplate(metaclass=ABCMeta):
    @property
    @abstractmethod
    def start_date(self):
        raise NotImplemented

    @property
    @abstractmethod
    def end_date(self):
        raise NotImplemented

    @property
    @abstractmethod
    def underlying(self):
        raise NotImplemented


class ModelParamTemplate(AbstractModelParamTemplate):
    def __init__(self):
        self.__start_date = None
        self.__end_date = None
        self.__underlyings = None

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return json.dumps(self.__dict__())

    def __dict__(self):
        return dict(start_date=self.__start_date,
                    end_date=self.__end_date,
                    underlyings=self.__underlyings)

    @property
    def start_date(self) -> Timestamp:
        return self.__start_date

    @property
    def end_date(self) -> Timestamp:
        return self.__end_date

    @property
    def underlying(self) -> dict:
        return self.__underlyings



import logging
import os
import boto3
from botocore.exceptions import ClientError
from io import StringIO
import pandas as pd

def set_location(local_file_path, folders=None):
    print('Expected format: [pair, day, hour] or similar')
    clean_f = []
    if folders is not None:
        for subfolder in folders:
            clean_f.append(subfolder.replace(' ', '_'))
        return '/'.join(clean_f) + '/' + os.path.basename(local_file_path)
    else:
        print('No subfolders found - file will be located in main')
        return os.path.basename(local_file_path)

class NapoleonS3Connector:
    """
    -- Methods included in boto --
​
    Create key inside a bucket:
        k = Key(self.resource.Bucket('bucket_name'))
​
    Set up key location (use randomize_key):
        k.key = 'location'
​
    Access existing key:
        k = self.resource.Bucket('bucket_name').get_key('key_name')
​
    Set contents from string:
        k.set_contents_from_string('string')
​
    Return content string:
        k.get_contents_as_string()
​
    Upload a file from local_path:
        k.set_contents_from_filename('local_path')
​
    Upload a local file into a bucket (one liner)
        self.resource.Bucket(bucket_name).upload_file(Filename='local_path', Key='your_key')
​
    Upload a local file into a bucket (alternatively):
        self.client.put_object(Bucket = 'bucket_name', Body = 'body', Key = 'my_key')
​
    Get contents to local_path:
        k.get_contents_to_filename('local_path')
​
    Get list of all keys in bucket:
        mybucket.list()
​
    Get list of all buckets (resource-greedy):
        self.client.get_all_buckets()
    """

    def __init__(self, aws_access_key_id, aws_secret_access_key, region=None):
        session = boto3.Session(aws_access_key_id, aws_secret_access_key)
        self.resource = session.resource('s3', region_name=region)
        self.client = session.client('s3', region_name=region)
        self.subfolders = None
        self.region = region

    def bucket_exist(self, bucket_name):
        return False if self.resource.Bucket(bucket_name).creation_date is None else True

    def create_bucket(self, bucket_name, region=None):
        region = self.region
        if region is None:
            try:
                self.client.create_bucket(Bucket=bucket_name)
            except ClientError as e:
                logging.error(e)
                return False
        else:
            location = {'LocationConstraint': region}
            self.client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration=location)
        print('-- Done -- ')

    def set_current_folder(self, folders):
        self.subfolders=folders

    def upload_content(self, bucket, local_file_path):
        '''
        if not bool(os.system("'FileChunkIO' in sys.modules")):
            raise SystemError('<< pip install FileChunkIO >> required')
​
        file_size = os.stat(local_file_path).st_size
        b = self.resource.Bucket(bucket)
        s3_file_name = set_location(local_file_path, self.subfolders)
        mp = b.initiate_multipart_upload(s3_file_name)
        chunk_count = int(math.ceil(file_size / float(max_chunk_size)))
        for i in range(chunk_count):
            offset = chunk_size * i
            bytes_ = min(chunk_size, source_size - offset)
            with FileChunkIO(local_file_path, 'r', offset=offset, bytes=bytes_) as fp:
                mp.upload_part_from_file(fp, part_num=i + 1)
        mp.complete_upload()
        '''
        key_location = set_location(local_file_path, self.subfolders)
        ExtraArgs = {'ContentType': os.path.splitext(local_file_path)[1], 'ACL': "public-read"}
        self.client.upload_file(local_file_path, bucket, key_location, ExtraArgs)
        print('-- Done uploading ' + os.path.basename(local_file_path) + ' @key ' + key_location)


    def download_content(self,bucket, filename):
        obj = self.client.get_object(Bucket=bucket, Key=filename)
        content = obj['Body'].read()
        #results = [r for r in obj['Body'].iter_lines()]
        #return results
        return content

    def download_dataframe_from_csv(self, bucket, filename, sep = ','):
        obj = self.client.get_object(Bucket=bucket, Key=filename)
        bytes_data_csv = obj['Body'].read()
        string_data_csv = StringIO(str(bytes_data_csv, 'utf-8'))
        dataframe = pd.read_csv(string_data_csv, sep=sep)
        return dataframe


    def upload_file(self, bucket, local_file_path):
        '''
        if not bool(os.system("'FileChunkIO' in sys.modules")):
            raise SystemError('<< pip install FileChunkIO >> required')
​
        file_size = os.stat(local_file_path).st_size
        b = self.resource.Bucket(bucket)
        s3_file_name = set_location(local_file_path, self.subfolders)
        mp = b.initiate_multipart_upload(s3_file_name)
        chunk_count = int(math.ceil(file_size / float(max_chunk_size)))
        for i in range(chunk_count):
            offset = chunk_size * i
            bytes_ = min(chunk_size, source_size - offset)
            with FileChunkIO(local_file_path, 'r', offset=offset, bytes=bytes_) as fp:
                mp.upload_part_from_file(fp, part_num=i + 1)
        mp.complete_upload()
        '''
        key_location = set_location(local_file_path, self.subfolders)
        ExtraArgs = {'ContentType': os.path.splitext(local_file_path)[1], 'ACL': "public-read"}
        self.client.upload_file(local_file_path, bucket, key_location, ExtraArgs)
        print('-- Done uploading ' + os.path.basename(local_file_path) + ' @key ' + key_location)

    def list_files(self, bucket):
        b = self.resource.Bucket(bucket)
        for key in b.objects.all():
            print(key)

    def download_file(self, bucket, file_name, target_local_path):
        self.client.download_file(bucket, set_location(file_name, self.subfolders), target_local_path)
        print('-- Done -- ')


__author__ = "hugo.inzirillo"

import json
from dataclasses import is_dataclass

import requests

from npxlogger import log


class Get(object):
    def __init__(self, flag: str):
        self.__flag = flag
        self.__header = dict()
        self.__connector = None

    @property
    def flag(self):
        return self.__flag

    @flag.setter
    def flag(self, flag):
        self.__flag = flag

    @property
    def header(self):
        return self.__header

    @header.setter
    def header(self, header):
        self.__header = header

    @property
    def connector(self):
        return self.__connector

    @connector.setter
    def connector(self, connector):
        self.__connector = connector

    def __call__(self, original_func):
        def wrappee(connector=None, **kwargs):
            self.connector = connector
            self.__fill_request(**kwargs)

            log.info("Api call rest end_point : {}".format(self.flag))
            log.info("header {}".format(self.header))
            return requests.get(url=self.flag, headers=self.header)

        return wrappee

    def __fill_request(self, **kwargs):
        if hasattr(self.connector, '__authentication_manager') and self.connector.authentication_manager is not None:
            self.header = self.connector.authentication_manager().authenticate().authorization_header
        else:
            self.header = None
        try:
            self.flag = self.flag.format(**kwargs)
        except Exception as e:
            log.error(e.args)


class Post(object):
    def __init__(self, flag: str):
        self.__flag = flag
        self.__header = dict()
        self.__connector = None

    @property
    def flag(self):
        return self.__flag

    @flag.setter
    def flag(self, flag):
        self.__flag = flag

    @property
    def header(self):
        return self.__header

    @header.setter
    def header(self, header):
        self.__header = header

    @property
    def connector(self):
        return self.__connector

    @connector.setter
    def connector(self, connector):
        self.__connector = connector

    def __call__(self, original_func):
        def wrappee(connector=None, **kwargs):

            try:
                self.connector = connector
                if "data" in kwargs:
                    data = self.__check_data(data=kwargs["data"])
                    self.__fill_request(**kwargs)
                    url = self.connector.connector_manager.url + self.flag
                    return self.__handle_response(response=requests.post(url=url, headers=self.header, data=data))
                else:
                    log.error("missing data in request {}".format(self.flag))
            except Exception as e:
                log.error(e.args)

        return wrappee

    def __check_data(self, data):
        if is_dataclass(data) and data is not None:
            try:
                data = data.__dict__
                return json.dumps(data)
            except RuntimeError as e:
                log.error(e.args)
                log.error("data {}".format(data))

    def __fill_request(self, **kwargs):
        if hasattr(self.connector.connector_manager,
                   'authentication_manager') and self.connector.connector_manager.authentication_manager is not None:
            self.header = self.connector.connector_manager.authentication_manager.authorization_header
        else:
            self.header = None
        try:
            self.flag = self.flag.format(**kwargs)
        except Exception as e:
            log.error(e.args)

    def __handle_response(self, response: requests.Response):
        try:
            if response.status_code == requests.status_codes.codes.OK:
                return response.json()
            elif response.status_code == requests.status_codes.codes.CREATED:
                return response
        except Exception as e:
            log.error(e.args)



__author__ = "hugo.inzirillo"

import requests

from napoleontoolbox.bifrost.models.auth import Scope, Token
from napoleontoolbox.bifrost.headers import AuthenticationManager, AuthorizationHeaderManager, \
    TemplateConnectorManager
from napoleontoolbox.bifrost.utils.auth_handler import AuthResponseHandlerTemplate
from napoleontoolbox.dataloader.models import NapoleonService
from napoleontoolbox.napoleon_config_tools import getter


class NapoleonServiceAuthHandler(AuthResponseHandlerTemplate):

    def _get_token(self):
        return self._response["access_token"]

    def _get_token_type(self):
        return str(self._response["token_type"]).capitalize()

    def get_token(self):
        _token = Token()
        _token.value = self._get_token()
        _token.type = self._get_token_type()
        return _token


class NapoleonAuthenticationManager(AuthenticationManager):

    @property
    @getter(value="providers.napoleon_service.access_token_uri")
    def url(self):
        return self.url

    @property
    def credentials(self):
        return NapoleonService().get_credentials()

    @property
    def scope(self):
        return Scope(["READ"])

    def _handle_reponse(self):
        if self.response:
            if self.response.status_code == requests.codes.OK:
                _reponse = self.response.json()
                self.token = NapoleonServiceAuthHandler(_reponse).get_token()
                self.authorization_header = AuthorizationHeaderManager(self.token).build()


class NapoleonConnectorManager(TemplateConnectorManager):
    def __init__(self):
        self.__authentication_manager = NapoleonAuthenticationManager()
        self.__url = str()
        self.authentication_manager.authenticate()

    @property
    def authentication_manager(self):
        return self.__authentication_manager

    @property
    def url(self) -> str:
        return self.__url

    @url.setter
    def url(self, url: str):
        self.__url = url



__author__ = "hugo.inzirillo"

from abc import abstractmethod, ABCMeta
from typing import Union

import requests.auth

from napoleontoolbox.bifrost.models.auth import ClientCredentials, PasswordCredentials, Token


class AbstractHeaderTemplate(metaclass=ABCMeta):

    @property
    @abstractmethod
    def value(self):
        raise NotImplemented

    @staticmethod
    def build(object: Union[PasswordCredentials, ClientCredentials, Token]):
        raise NotImplemented


class AbstractHeaders(AbstractHeaderTemplate):

    @property
    @abstractmethod
    def value(self):
        raise NotImplemented

    @staticmethod
    def build(object: Union[PasswordCredentials, ClientCredentials, Token]):
        raise NotImplemented


class AbstractHeadersScheme(AbstractHeaderTemplate):

    @property
    @abstractmethod
    def value(self):
        raise NotImplemented

    @classmethod
    def build(self, object: Union[PasswordCredentials, ClientCredentials, Token]):
        raise NotImplemented


class HeadersScheme(AbstractHeaderTemplate):

    def __init__(self, o: Union[Token] = None):
        self.__object = o
        self.__header = None
        self.__auth = requests.auth.HTTPBasicAuth

    @property
    def object(self):
        return self.__object

    @property
    def value(self):
        return self.__header

    @value.setter
    def value(self, value: dict):
        self.__header = value

    def build(self):
        return self


class AuthorizationHeaderScheme(HeadersScheme):
    def build(self):
        if isinstance(self.object, Token):
            self.value = {
                "Authorization": "{type} {token_value}".format(type=self.object.type, token_value=self.object.value),
                'Content-Type': 'application/json',
                'Accept': 'application/json'
            }
        return self


class JsonHeaderScheme(HeadersScheme):
    def build(self):
        self.value = {
            "Accept": "application/json",
            "Content-Type": "application/x-www-form-urlencoded;charset=UTF-8",
        }
        return self


class TemplateHeaders(AbstractHeaders):

    def value(self): ...

    @staticmethod
    def build(object: Union[PasswordCredentials, ClientCredentials, Token]): ...


class Headers(TemplateHeaders):
    def __init__(self):
        self.__value = dict()

    @property
    def value(self) -> dict:
        return self.__value

    @value.setter
    def value(self, x) -> dict:
        self.__value = x

    def __dict__(self):
        return self.__value

    @staticmethod
    def build(o: Union[Token] = None):
        return HeaderGenerator().build(o)


class HeaderGenerator(Headers):

    def build(self, object: Union[Token] = None):

        if object:
            if isinstance(object, Token):
                self.value = AuthorizationHeaderScheme(object).build().value

        else:
            self.value = JsonHeaderScheme().build().value
        return self.value


class ApplicationJsonHeader(Headers):
    pass


class AuthorizationHeader(Headers):
    pass
